@@ -4,7 +4,7 @@ use super::iocost_qos::{
 };
 use super::protection::MemHog;
 use super::*;
-use statrs::distribution::{Normal, Univariate};
+use statrs::distribution::{ContinuousCDF, Normal};
 use std::cell::RefCell;
 use std::cmp::{Ordering, PartialOrd};
 use std::collections::{BTreeMap, BTreeSet};
@@ -382,8 +382,9 @@ enum QoSTarget {
     VrateRange((f64, f64), (Option<String>, Option<String>)),
     MOFMax,
     AMOFMax,
+    AMOFMaxVrate,
+    AMOFDeltaMin,
     IsolatedBandwidth,
-    Isolation,
     LatRange(DataSel, (f64, f64)),
 }
 
@@ -415,10 +416,11 @@ impl std::fmt::Display for QoSTarget {
             }
             Self::MOFMax => write!(f, "MOF=max").unwrap(),
             Self::AMOFMax => write!(f, "aMOF=max").unwrap(),
+            Self::AMOFMaxVrate => write!(f, "aMOF=max-vrate").unwrap(),
+            Self::AMOFDeltaMin => write!(f, "aMOF-delta=min").unwrap(),
             Self::IsolatedBandwidth => {
-                write!(f, "isolated-bandwidth (max(isolation, aMOF=MAX))").unwrap()
+                write!(f, "(lat-imp=min).clamp(isolation, bandwidth)").unwrap()
             }
-            Self::Isolation => write!(f, "isolation (aMOF-delta=min)").unwrap(),
             Self::LatRange(sel, (low, high)) => match sel {
                 DataSel::RLat(lat_pct, _) => {
                     write!(f, "rlat-{}={}-{}", lat_pct, low, high).unwrap()
@@ -507,20 +509,22 @@ impl QoSTarget {
         let v = v.to_lowercase();
         match k.as_str() {
             "isolated-bandwidth" => Ok(Self::IsolatedBandwidth),
-            "isolation" => Ok(Self::Isolation),
             k => {
                 let sel = DataSel::parse(k)?;
                 match &sel {
-                    DataSel::MOF | DataSel::AMOF => {
-                        if v != "max" {
-                            bail!("Invalid {:?} value {:?}", &sel, &v);
-                        }
-                        if sel == DataSel::MOF {
-                            Ok(Self::MOFMax)
-                        } else {
-                            Ok(Self::AMOFMax)
-                        }
-                    }
+                    DataSel::MOF => match v.as_str() {
+                        "max" => Ok(Self::MOFMax),
+                        v => bail!("Invalid {:?} value {:?}", &sel, &v),
+                    },
+                    DataSel::AMOF => match v.as_str() {
+                        "max" => Ok(Self::AMOFMax),
+                        "max-vrate" => Ok(Self::AMOFMaxVrate),
+                        v => bail!("Invalid {:?} value {:?}", &sel, &v),
+                    },
+                    DataSel::AMOFDelta => match v.as_str() {
+                        "min" => Ok(Self::AMOFDeltaMin),
+                        v => bail!("Invalid {:?} value {:?}", &sel, &v),
+                    },
                     DataSel::RLat(_, time_pct) | DataSel::WLat(_, time_pct) => {
                         if time_pct != "mean" {
                             bail!("Latency range target should have \"mean\" for time percentile");
@@ -562,13 +566,9 @@ impl QoSTarget {
             }
             Self::MOFMax => vec![DataSel::MOF, DataSel::LatImp],
             Self::AMOFMax => vec![DataSel::AMOF, DataSel::LatImp],
-            Self::IsolatedBandwidth => vec![
-                DataSel::MOF,
-                DataSel::AMOF,
-                DataSel::LatImp,
-                DataSel::AMOFDelta,
-            ],
-            Self::Isolation => vec![DataSel::MOF, DataSel::LatImp, DataSel::AMOFDelta],
+            Self::AMOFMaxVrate => vec![DataSel::AMOF],
+            Self::AMOFDeltaMin => vec![DataSel::AMOFDelta],
+            Self::IsolatedBandwidth => vec![DataSel::LatImp, DataSel::AMOF, DataSel::AMOFDelta],
             Self::LatRange(sel, _) => vec![sel.clone()],
         }
     }
@@ -713,7 +713,9 @@ impl QoSTarget {
             )
         };
 
-        let solve_mof_max = |sel, no_sig_sel| -> Result<Option<f64>> {
+        // Find min vrate at max val for @sel. If the line is flat, use the
+        // min vrate at max val for @no_sig_sel.
+        let solve_max = |sel, no_sig_sel| -> Result<Option<f64>> {
             let no_sig_vrate = match no_sig_sel {
                 Some(nssel) => Self::find_max_vrate_at_min_val(
                     ds(nssel)?,
@@ -730,18 +732,21 @@ impl QoSTarget {
             ))
         };
 
-        let solve_isolation = || -> Result<Option<f64>> {
-            let amof_delta_ds = ds(&DataSel::AMOFDelta)?;
-            Ok(solve_mof_max(&DataSel::MOF, Some(&DataSel::LatImp))?
-                .map(|mof_max| {
-                    Self::find_max_vrate_at_min_val(
-                        amof_delta_ds,
-                        (scale_min, mof_max),
-                        infl_offset(),
-                    )
-                })
-                .filter(|vrate| vrate.is_some())
-                .map(|vrate| vrate.unwrap()))
+        // Find the max vrate at min val.
+        let solve_min = |sel| -> Result<Option<f64>> {
+            Ok(Self::find_max_vrate_at_min_val(
+                ds(sel)?,
+                (scale_min, scale_max),
+                infl_offset(),
+            ))
+        };
+
+        // Find the rightmost valid vrate.
+        let solve_max_vrate = |sel| -> Result<Option<f64>> {
+            Ok(ds(sel)?
+                .lines
+                .clamped(scale_min, scale_max)
+                .map(|dl| dl.range.1))
         };
 
         Ok(match self {
@@ -761,28 +766,33 @@ impl QoSTarget {
                     *scale_max,
                 ))
             }
-            Self::MOFMax => {
-                solve_mof_max(&DataSel::MOF, Some(&DataSel::LatImp))?.map(params_at_vrate)
-            }
+
+            // Min vrate still at max MOF. If MOF is flat, max vrate at min
+            // LatImp.
+            Self::MOFMax => solve_max(&DataSel::MOF, Some(&DataSel::LatImp))?.map(params_at_vrate),
+
+            // Min vrate still at max aMOF. If MOF is flat, max vrate at min
+            // LatImp.
             Self::AMOFMax => {
-                solve_mof_max(&DataSel::AMOF, Some(&DataSel::LatImp))?.map(params_at_vrate)
+                solve_max(&DataSel::AMOF, Some(&DataSel::LatImp))?.map(params_at_vrate)
             }
-            Self::IsolatedBandwidth => {
-                // Isolation if it has the maximum aMOF; otherwise, the
-                // lowest vrate which maps to max aMOF. Note that we don't
-                // want to consider LatImp when determining aMOF maximum
-                // point here - we're already pushing higher than the
-                // Isolation solution and don't want to go any higher than
-                // needed for reaching the aMOF max point.
-                match (solve_mof_max(&DataSel::AMOF, None)?, solve_isolation()?) {
-                    (None, None) => None,
-                    (amof_max, isolation) => {
-                        Some(amof_max.unwrap_or(0.0).max(isolation.unwrap_or(0.0)))
-                    }
+
+            // Rightmost vrate with valid aMOF.
+            Self::AMOFMaxVrate => solve_max_vrate(&DataSel::AMOF)?.map(params_at_vrate),
+
+            // clamp(max vrate at min LatImp, isolation, bandwidth)
+            Self::IsolatedBandwidth => match (
+                solve_min(&DataSel::AMOFDelta)?,
+                solve_max_vrate(&DataSel::AMOF)?,
+            ) {
+                (Some(min), Some(max)) => {
+                    solve_min(&DataSel::LatImp)?.map(|v| params_at_vrate(v.clamp(min, max)))
                 }
-                .map(params_at_vrate)
-            }
-            Self::Isolation => solve_isolation()?.map(params_at_vrate),
+                _ => None,
+            },
+
+            Self::AMOFDeltaMin => solve_min(&DataSel::AMOFDelta)?.map(params_at_vrate),
+
             Self::LatRange(sel, lat_rel_range) => {
                 if let Some((lat_target, vrate_range)) =
                     Self::solve_lat_range(ds(&sel)?, *lat_rel_range, (scale_min, scale_max))
@@ -926,9 +936,9 @@ impl Bench for IoCostTuneBench {
             };
 
             push_props(&[("name", "naive")]);
-            push_props(&[("name", "bandwidth"), ("mof", "max")]);
+            push_props(&[("name", "bandwidth"), ("amof", "max-vrate")]);
             push_props(&[("name", "isolated-bandwidth"), ("isolated-bandwidth", "")]);
-            push_props(&[("name", "isolation"), ("isolation", "")]);
+            push_props(&[("name", "isolation"), ("amof-delta", "min")]);
             push_props(&[("name", "rlat-99-q1"), ("rlat-99", "q1")]);
             push_props(&[("name", "rlat-99-q2"), ("rlat-99", "q2")]);
             push_props(&[("name", "rlat-99-q3"), ("rlat-99", "q3")]);
@@ -1694,7 +1704,9 @@ impl IoCostTuneJob {
         // Remark on aMOF-delta.
         for (name, sol) in res.solutions.iter() {
             match &sol.target {
-                QoSTarget::IsolatedBandwidth | QoSTarget::Isolation => {
+                QoSTarget::AMOFMaxVrate
+                | QoSTarget::AMOFDeltaMin
+                | QoSTarget::IsolatedBandwidth => {
                     let err = sol.adjusted_mem_offload_delta / sol.mem_offload_factor;
                     if err >= 0.05 {
                         remarks.push(format!(
