@@ -643,7 +643,7 @@ impl<'a, 'b> RunCtx<'a, 'b> {
         let started_at = unix_now() as i64;
         if let Err(e) = self.wait_cond(
             |af, progress| {
-                progress.set_status("Waiting rd-agent to start");
+                progress.set_status("Waiting for rd-agent to start");
                 let rep = &af.report.data;
                 rep.timestamp.timestamp() > started_at && rep.state == RunnerState::Running
             },
@@ -913,8 +913,6 @@ impl<'a, 'b> RunCtx<'a, 'b> {
         Ok(())
     }
 
-    pub const BENCH_FAKE_CPU_RPS_MAX: u32 = 2000;
-
     pub fn start_hashd_bench(
         &mut self,
         log_bps: Option<u64>,
